#!/bin/bash

# This helper script will handle deploying custom terraform
set -e

source /scripts/utils.sh
source /scripts/terraform.sh

######
# Main
######

cprint "Displaying environment variables after TF setup"
printenv

tfAdditionalVarsFile=${tfRoot}/additional_settings.auto.tfvars

# Write environment variables coming from helm chart to TF variables file
if [ ! -z "${extraVar}" ]; then
  # Check for early exit
  deploy_infra=$(echo $vars | jq -r .DEPLOY_INFRA)
  if [ "$deploy_infra" = "false" ]; then
    cprint "Exiting early. Infra deployment skip detected." "red"
    echo "Printing vars:"
    echo $vars | jq
    exit 0
  fi

  # Exclude anything that shouldn't be a TF variable
  tf_vars=$(echo $vars | jq 'del(.GLUE_HANDLER_DEPLOY_MAP, .DEPLOY_INRFA)' | jq -r tostring)

  echo "extra_environment = ${tf_vars}" >> ${tfAdditionalVarsFile}
fi

# Pull out the ARN of the SQS DLQ
# TODO: Remove this once MOSAIC20-2553 is done
job_dlq_arn=$(kubectl get configmap nta-pop-infra-nta-pop-filter-sqs-arn  -o jsonpath="{.data.ARN}")
echo "job_dlq_arn = \"${job_dlq_arn}\"" >> ${tfAdditionalVarsFile}
event_bridge_dlq_arn=$(kubectl get configmap nta-pop-infra-nta-pop-filter-dlq-arn -o jsonpath="{.data.ARN}")
echo "event_bridge_dlq_arn = \"${event_bridge_dlq_arn}\"" >> ${tfAdditionalVarsFile}

# Prep for artifact deployment
cprint "Creating glue handler deployment map"
export GLUE_HANDLER_DEPLOY_MAP="/artifacts/glue-handler-deployment.json"
echo $vars | jq .GLUE_HANDLER_DEPLOY_MAP > $GLUE_HANDLER_DEPLOY_MAP
echo "Created the following glue handler deployment map:"
jq . $GLUE_HANDLER_DEPLOY_MAP

# Add naming prefix
echo "name_prefix = \"$externalNamingPrefix\"" >> ${tfAdditionalVarsFile}

# Must initialize TF first, which sets up the AWS config
terraform_init

#####
# Setup IAM
#
# The environments are defined in the core-infrastructure repo
# Refer to the tfvars file for each account and specifically the account_mosaic_env_list variable
# Example for confer-test: https://gitlab.bit9.local/mcs2/core-infrastructure/-/blob/master/core-infra/accounts/confer-test/variables.tfvars#L170
# It is expected that the created role will already be pre-defined to be one of those values
#
# This complicated logic is to allow a mismatched IAM role.
# By default, mcs2 assumes that the name of the role will match the ENV variable.
# But the ENV variable for dev and team clusters will never match the role name.
# The approach taken here is to default to ENV and fallback on dev.
# Dev is only used in the event that the account is confer-test or cb-dev.
#
# If the resulting resource (IAM role) doesn't exist, the TF will fail.
# Validation does not occur directly in bash.
#
# This does not support all cases, but rather the necessary ones for this use case.
#
# The end result should be a relatively safe operation to allow one IAM role in
# dev to be used by all dev clusters without requiring any additional configuration.
#####

# Default with assuming the role exists
mcs2_env="${env}"

# Extract out variables
iam_role_name=$(echo $vars | jq -r .IAM_ROLE_NAME)

# Check if the IAM role does not exist
env_role_exist="$(aws iam get-role \
    --role-name mcs2-app-${mcs2_env}-${iam_role_name}-role \
    2>&1 >/dev/null | grep -c NoSuchEntity)" || true

if [ $env_role_exist -eq 1 ]; then
    # Role doesn't exist, so grab account alias
    # This assumes a single alias
    account_alias="$(aws iam list-account-aliases \
        --output json | jq -r .AccountAliases[0])"

    # Verify if this is a supported dev environment
    if [ "$account_alias" = "confer-test" ]; then
        # Valid environment, map to dev
        # No need to verify account existence, b/c TF does it
        mcs2_env="kcdev01"
    elif [ "$account_alias" = "cb-dev" ] || [ "$account_alias" = "carbonblack-dev" ]; then
        mcs2_env="partial"
    else
        error_exit "Account alias \"${account_alias}\" is not a valid dev account!"
    fi
fi

full_iam_name="mcs2-app-${mcs2_env}-${iam_role_name}-role"
echo "Using IAM role \"$full_iam_name\""
echo "iam_role_name = \"$full_iam_name\"" >> ${tfAdditionalVarsFile}

# To override unknown test environment, such your personal ones, like yivan1.
echo "mcs2_env = \"$mcs2_env\"" >> ${tfAdditionalVarsFile}

cprint "Displaying TF variables"
run_cmd cat "${tfAdditionalVarsFile}"

cprint "Displaying environment variables after init script execution"
printenv

print_header "Initialization completed"
